"""
Shared Library Runtime module

Loads `libin3` according to host hardware architecture and OS.
Maps symbols, methods and types from the library.
Encapsulates low-level rpc calls into a comprehensive runtime.
"""
