"""
Integrated tests for `in3`

Tests data marshalling from and to libin3.
Tests minimum, happy path, client consistency and stability.
Calls all implemented calls from Python to libin3.
Uses real network data mocked locally.
"""
